@@ -13,66 +13,16 @@ are assumed to be the HII regions when using a H alpha map.
 Citation: Easeman et al. (2022), in prep.
 """
 
-from os import path
 import sys
 import numpy as np
 
-
-from MUSEspec.spectrum3d import Spectrum3d
-
 from astropy.io import fits
 from astropy.cosmology import WMAP9 as cosmo #pylint: disable=no-name-in-module
 
 
 
 
-def read_in_map(fname, SNlim=None, verbose=False, BPT_cut=True, z=None, datadir='./',
-galaxy_name=''):
-    """
-    Reads in a fits file from a given filename, and returns the data and header
 
-    fname - str
-        The relative filepath to the required map (normally H alpha)
-    SNlim - float
-        If a float is given, the flux values for pixels with a signal / noise of less
-        than this value are set to 0. Default = None.
-    verbose - bool
-        If True, prints additional information. Default = False.
-    =============
-    Returns:
-        Map, header
-    """
-    print("Reading in ", fname)
-    if not path.isfile(fname): #check file exists
-        raise Exception(f"File {fname} not found.")
-
-    with fits.open(fname) as tmp:
-        data, header = tmp['MAP'].data, tmp['MAP'].header # pylint: disable=no-member
-        # disabling pylint error for no-member, as it's suggesting an issue with
-        # .data and .header for some reason!
-        if SNlim is not None:
-            std = tmp['STD'].data # pylint: disable=no-member
-            # Set flux values to 0. for pixels with S/N < the given limit.
-            sel = (data/std) < SNlim
-            data[sel] = 0.
-
-            print_msg(f"{np.count_nonzero(sel)} pixels removed due to S/N lim", verbose)
-
-
-    if BPT_cut:
-        if z is None or galaxy_name=='':
-            raise Exception("read_in_map: need to specify a redshift and galaxy name if want\
-                             to apply BPTcut")
-        s3d = Spectrum3d(filen = datadir+galaxy_name+'_gas_cube.fits.fz', datadir=datadir,
-                        target=galaxy_name, z=z, fwhm=1)
-
-        SF_sel = np.array(s3d.BPTcut(return_line_ratios=False))
-        # Returns boolean mask, with True for SF pixels
-        data[~SF_sel] = np.nan
-
-
-    print_msg('Map read in successfully', verbose)
-    return data, header, (data/std)
 
 
 def determine_bkg_flux(linemap, HaEW, distmap, hasn, hasn_lim, \
@@ -98,7 +48,7 @@ def determine_bkg_flux(linemap, HaEW, distmap, hasn, hasn_lim, \
 
 def identify_HII_regions(linemap, header, flux_llim, flux_ulim, z, #pylint: disable=invalid-name
                          obj_name, bkg_flux, max_radius_kpc=None, max_radius_arcsec=None,
-                         min_pixels=2, verbose=False, tdir='./'):
+                         min_pixels=2, min_separation=0., verbose=False, tdir='./'):
     # getting pylint to ignore that HII doesn't conform to snake-case style
     """
     Using a Ha map, identifies HII regions based on the brightest regions within the map.
@@ -155,6 +105,9 @@ def identify_HII_regions(linemap, header, flux_llim, flux_ulim, z, #pylint: disa
  limit of {flux_ulim:.2e}")
     linemap[linemap > flux_ulim] = 0.
     num_bright_pixels = np.count_nonzero(linemap > flux_llim)
+    if num_bright_pixels == 0:
+        raise Exception(f"No pixels with flux above {flux_llim} identified in the map.")
+
     print(f"{num_bright_pixels} pixels have flux values above the given flux_llim of \
 {flux_llim:.2e}")
 
@@ -193,19 +146,28 @@ HII regions.")
 
         # Check that the surrounding 8 pixels meet various conditions
         linemap, peak_tracker_val, reject_msg = check_surrounding_pixels(linemap, peak_idx,\
-                                                peak_tracker_val=peak_tracker[peak_idx])
+                                                            peak_tracker_val=peak_tracker[peak_idx])
         peak_tracker[peak_idx] = peak_tracker_val
         if linemap[peak_idx] == -99: #if peak is rejected
             print_msg(reject_msg, verbose)
             continue
 
 
-        # Have identified a peak, now want to check the surrounding pixels, and add them to the
-        # HII region
+        # Have identified a peak, now want to add surrounding pixels to the HII region
 
-        #Start by determining annuli to use
+        #Start by determining maps of distance from the peak, and annuli to use
         annuli, distmap = determine_annuli(yindx,xindx,peak_idx,maxdist, annulus_ind, AngD, pixsky)
 
+
+        #Check that there isn't another peak closer than the min_separation away
+        linemap, peak_tracker_val, reject_msg = check_nearest_peak(linemap, peak_idx,\
+            peak_tracker_val=peak_tracker[peak_idx], distmap=distmap, min_separation=min_separation)
+        peak_tracker[peak_idx] = peak_tracker_val
+        if linemap[peak_idx] == -99: #if peak is rejected
+            print_msg(reject_msg, verbose)
+            continue
+
+
         # Want to go through each pixel within each annulus, checking if they meet the criteria to
         # be added to the HII region. Will then check if the number of pixels meeting the
         # criteria meets the minimum number of pixels specified in the function call.
@@ -218,7 +180,7 @@ HII regions.")
 
         if len(add_to_region) > min_pixels:
             peaks_map[peak_idx] = region_ID
-            peak_tracker[peak_idx] = 4.
+            peak_tracker[peak_idx] = 5.
             for idx, dist in zip(add_to_region, distance):
                 region_ID_map[idx] = region_ID
                 dist_from_peak[idx] = dist
@@ -258,11 +220,12 @@ pixels assigned to a HII region \n")
     print("\nMaps saved at:  ", tdir+obj_name+"_HII_segmentation_map.fits   ", \
         tdir+obj_name+"_HII_trackermaps.fits")
 
-    print("\n==============\n Peaks tracker map: \n 0: Pixel not considered \
+    print(f"\n==============\n Peaks tracker map: \n 0: Pixel not considered \
     \n 1: Surrounded by >4 non-finite values - just noise. \
     \n 2: Adjacent to an already-identified peak, or pixel with higher flux \
     \n 3: Median of surrounding pixels < 0.1 * peak_flux \
-    \n 4: Pixel successfully identified as the peak of a HII region. \
+    \n 4: Within {min_separation}kpc of another peak \
+    \n 5: Pixel successfully identified as the peak of a HII region. \
     \n==============\n Pixel tracker map: \n 0: Pixel not considered \
     \n 1: Pixel rejected - below flux threshold \
     \n 2: Pixel assigned to region \n 3: Pixel has moved region \
@@ -296,7 +259,7 @@ def determine_distances(z, max_radius_arcsec, max_radius_kpc, header, linemap_sh
     ==========
     """
     #Determine the maximum distance from the peak to consider pixels for HII region
-    AngD=(cosmo.kpc_proper_per_arcmin(z).value[0]) /60
+    AngD=(cosmo.kpc_proper_per_arcmin(z).value) /60
     pixsky = (header['CD1_1']**2 + header['CD1_2']**2) ** 0.5 * 3600
 
     # arcsec to kpc - multiply by AngD
@@ -350,8 +313,13 @@ def check_surrounding_pixels(linemap, peak_idx, peak_tracker_val):
     reject_msg: str, message describing reason for rejection
     """
 
-    surrounding_ha = linemap[peak_idx[0]-1:peak_idx[0]+2, peak_idx[1]-1:peak_idx[1]+2].flatten()
-    surrounding_ha = np.delete(surrounding_ha, 4) # remove central
+
+    yindx, xindx = np.indices(linemap.shape)[0], np.indices(linemap.shape)[1]
+    ymin, ymax = peak_idx[0]-1, peak_idx[0]+2
+    xmin, xmax = peak_idx[1]-1, peak_idx[1]+2
+    mask = (yindx < ymax) & (yindx >= ymin) & (xindx >= xmin) & (xindx < xmax)
+    mask[peak_idx] = False # remove the central pixel
+    surrounding_ha = linemap[mask]
 
     if np.count_nonzero(~np.isfinite(surrounding_ha)) > 4:
         reject_msg = "Bright pixel is surrounded by >4 non-finite values"
@@ -378,6 +346,26 @@ disregarding this one"
 
 
 
+def check_nearest_peak(linemap, peak_idx, peak_tracker_val, distmap, min_separation):
+    """
+    Checks if another peak has already been identified less than min_separation away.
+    """
+
+    if np.any(distmap[linemap == -99] < min_separation):
+        reject_msg = f"Within {min_separation}kpc of another peak"
+        peak_tracker_val = 4.
+        linemap[peak_idx] = -99
+
+    else:
+        reject_msg=''
+        peak_tracker_val = -5
+
+
+    return linemap, peak_tracker_val, reject_msg
+
+
+
+
 def determine_annuli(yindx, xindx, peak_idx, maxdist, annulus_ind, AngD, pixsky):
     """
     Determine the indices of pixels falling within each annulus, moving outwards
@@ -427,70 +415,6 @@ def determine_annuli(yindx, xindx, peak_idx, maxdist, annulus_ind, AngD, pixsky)
 
 
 
-# def add_pixels_to_region(annuli, linemap, peak_idx, peak_flux, flux_fraction, pixel_tracker,
-# distmap, dist_from_peak, verbose, region_ID_map):
-#    """
-#     Determines which pixels meet the conditions to be added to the region. moving radially
-#     outwards from the peak, in annuli.
-
-#     ==========
-#     Returns:
-#     ==========
-#     add_to_region: list of indices of pixels to be added to the region
-#     distance: distance of each pixel from the peak
-#     moved_pixels: list of indices of pixels which have moved region, to be marked in the pixel
-#      tracker map
-#     """
-
-#     add_to_region, distance, moved_pixels = [peak_idx],[0.], []
-#     med_prev_annulus = 1e100 # Want medians to get lower as move outwards
-
-#     for annulus in annuli:
-#         num_in_annulus, num_below_lim = len(annulus), 0
-
-#         annulus_fluxes = np.array([linemap[y,x] for y,x in annulus])
-#         already_assigned = np.array([region_ID_map[y,x]>0 for y,x in annulus])
-
-#         if np.nanmedian(annulus_fluxes[~already_assigned]) > med_prev_annulus:
-#             break # Stop if median has gone down and then up again
-#             #Only consider unassigned pixels to avoid issues with merged regions.
-
-
-#         for y,x in annulus: # Go through each pixel in the annulus
-
-#             # If the flux is below peak flux * flux fraction, then reject
-#             if not (linemap[y,x] >= (peak_flux * flux_fraction)): #so NaNs also excluded
-#                 #y,x as MUSE cubes in the form y,x,lambda when read in using fits.open().
-#                 num_below_lim += 1
-#                 if pixel_tracker[y,x] <= 0: #if hasn't already been assigned to region
-#                     pixel_tracker[y,x] = 1.
-#                 continue
-
-#             # Otherwise...
-#             if region_ID_map[y,x] == -50:
-#                 # If pixel hasn't been assigned to a region already.
-#                 add_to_region.append((y,x))
-#                 distance.append(distmap[y,x])
-#             else:
-#                 print(f"Pixel ({x},{y}) has already been assigned to a region", verbose)
-#                 if dist_from_peak[y,x] <= distmap[y,x]:
-#                     #If closer to previous peak, leave there
-#                     print_msg("Closer to previous peak, leaving where it is.", verbose)
-#                 else:
-#                     print_msg("Moving to new region", verbose)
-#                     moved_pixels.append((y,x))
-#                     add_to_region.append((y,x))
-#                     distance.append(distmap[y,x])
-
-#         if num_below_lim == num_in_annulus: # If all pixels in that annulus
-#             # were below the threshold, then stop there.
-#             print_msg("Finished adding pixels to region", verbose)
-#             break
-#         med_prev_annulus = np.nanmedian(annulus_fluxes)
-
-#     return add_to_region, distance, moved_pixels
-
-
 def add_pixels_to_region(annuli, linemap, peak_idx, peak_flux, bkg_flux, pixel_tracker, distmap,
                          dist_from_peak, verbose, region_ID_map, isolated_pixel_limit=50):
     """
